//  cards.cs - 
//     Blackjack 
//--------------------------------------------------------------------

using System;

public enum CardSuit
{
     Zero_Error,
     clubs,
     diamonds,
     hearts,
     spades
}

public enum CardValue
{
     Zero_Error,
     Ace,
     two,
     three,
     four,
     five,
     six,
     seven,
     eight,
     nine,
     ten,
     Jack,
     Queen,
     King
}    

// Structure: card
//==============================================
struct card
{
    public CardSuit suit;  // 1 - 4
    public CardValue val;  // 1 - 13

    public int CardValue
    {
       get
       {
          int retval;

          if( (int) this.val >= 10)
             retval = 10;
          else
          if( (int) this.val == 1 )
             retval = 11;
          else
             retval = (int) this.val;

          return retval;
       }
    }

    public override string ToString()
    {
        return (string.Format("{0} of {1}", this.val.ToString("G"),
                                            this.suit.ToString("G")));
    }
}

// Class: deck
//==============================================
class deck
{
   public card [] cards = new card[53] ;
   int next;

   // deck()
   //   Constructor for setting up a regular deck
   //==============================================
   public deck()
   {
      next = 1;   // initialize pointer to point to first card.

      // Initialize the cards in the deck
      cards[0].val = 0;   // card 0 is set to 0.      
      cards[0].suit = 0;
      
      int currcard = 0;
      for( int suitctr = 1; suitctr < 5; suitctr++ )
      {
         for( int valctr = 1; valctr < 14; valctr++ )    
         {
             currcard = (valctr) + ((suitctr - 1) * 13);
             cards[currcard].val = (CardValue) valctr;
             cards[currcard].suit  = (CardSuit) suitctr;
         } 
      }
   }

   // shuffle()
   //   Randomizes a deck's cards
   //==============================================
   public void shuffle()
   {
      Random rnd = new Random();
      int sort1;
      int sort2;
      card tmpcard = new card();
      
      for( int ctr = 0; ctr < 100; ctr++)
      {
         sort1 = (int) ((rnd.NextDouble() * 52) + 1);
         sort2 = (int) ((rnd.NextDouble() * 52) + 1);

         tmpcard = this.cards[sort1];
         this.cards[sort1] = this.cards[sort2];
         this.cards[sort2] = tmpcard;
      }   
      
      this.next = 1;  // reset pointer to first card
   }

   // dealCard()
   //   Returns next card in deck
   //==============================================
   public card dealCard()
   {  
      if( next > 52 )
      {
          // At end of deck
          return (this.cards[0]);
      }
      else
      {
          // Returns current card and increments next
          return this.cards[next++];
      }
   }
}

// Class: CardGame
//==============================================

class CardGame
{
   static deck mydeck = new deck();
   static card [] pHand = new card[10];
   static card [] cHand = new card[10];
          
   public static void Main()
   {
      int pCardCtr = 0;
      int pTotal = 0;
      int cTotal = 0;
      
      bool playing = true;

      while ( playing == true )
      {
         //CLEAR HANDS
         pTotal = 0;
         cTotal = 0;
         pCardCtr = 0;

         for ( int ctr = 0; ctr < 10; ctr++)
         {
            pHand[ctr].val = 0;
            pHand[ctr].suit = 0;
         }
         
         Console.WriteLine("\nShuffling cards...");
         mydeck.shuffle();

         Console.WriteLine("Dealing cards...");

         pHand[0] = mydeck.dealCard();
         cHand[0] = mydeck.dealCard();
         pHand[1] = mydeck.dealCard();
         cHand[1] = mydeck.dealCard();
         
         // Set computer total equal to its first card...
         cTotal = cHand[0].CardValue; 

         bool playersTurn = true;
         
         do
         {         
            Console.WriteLine("\nPlayer\'s Hand:");
            pCardCtr = 0;
            pTotal = 0;

            do         
            {                
                Console.WriteLine("  Card {0}:  {1}", 
                           pCardCtr + 1,
                           pHand[pCardCtr].ToString());

                // Add card value to player total
                pTotal += pHand[pCardCtr].CardValue;

                pCardCtr++;
                   
            } while ((int) pHand[pCardCtr].val != 0);

            Console.WriteLine("Dealer\'s Hand:");

            Console.WriteLine("  Card 1:  {0}", 
                        cHand[0].ToString());
             

            Console.WriteLine("----------------------------");
            Console.WriteLine("Player Total = {0} \nDealer Total = {1}",
                              pTotal, cTotal);
         

            if( pTotal <= 21 )
            {
               playersTurn = GetPlayerOption(pCardCtr);
            }
            else
            {
               playersTurn = false;
            }

         } while(playersTurn == true);

         // Player's turn is done

         if ( pTotal > 21 )
         {
             Console.WriteLine("\n\n**** BUSTED ****\n");
         }
         else // Determine computer's score
         {
            // Tally Computer's current total...
            cTotal += cHand[1].CardValue;

            int cCardCtr = 2;

            Console.WriteLine("\n\nPlayer\'s Total:  {0}", pTotal);
            Console.WriteLine("\nComputer: ");
            Console.WriteLine("  {0}", cHand[0].ToString());
            Console.WriteLine("  {0} TOTAL: {1}",
                                  cHand[1].ToString(),
                                  cTotal);
                                               
            while ( cTotal < 17 )  // Less than 17, must draw
            {
               cHand[cCardCtr] = mydeck.dealCard();
               cTotal += cHand[cCardCtr].CardValue;
               Console.WriteLine("  {0} TOTAL: {1}",
                                    cHand[cCardCtr].ToString(),
                                    cTotal);
               cCardCtr++;        
            }

            if (cTotal > 21 )
            {
               Console.WriteLine("\n\nComputer Busted!");
               Console.WriteLine("YOU WON!!!");
            }
            else
            {
               if( pTotal > cTotal)
               {
                  Console.WriteLine("\n\nYOU WON!!!");
               }
               else
               if( pTotal == cTotal )
               {
                  Console.WriteLine("\n\nIt\'s a push");
               }
               else
               {
                  Console.WriteLine("\n\nSorry, The Computer won");
               }
            }
         }

         Console.Write("\n\nDo you want to play again?  ");
         string answer = Console.ReadLine();

         try
         {
            if( answer[0] != 'y' && answer[0] != 'Y' )
            {
               //Quiting
               playing = false;  
            }
         }
         catch( System.IndexOutOfRangeException )
         {
            // Didn't enter a value so quit
            playing = false;
         }
      }
   }

   // GetPlayerOption()
   //   Returns true to hit, false to stay
   //==============================================

   static bool GetPlayerOption( int cardctr )
   {
      string buffer;
      bool cont = true;
      bool retval = true;

      while(cont == true)
      {
         Console.Write("\n\nH = Hit, S = Stay ");
         buffer = Console.ReadLine();

         try
         {
            if ( buffer[0] == 'h' || buffer[0] == 'H')
            {
                pHand[cardctr] = mydeck.dealCard();
                cont = false;
            }
            else if( buffer[0] == 's' || buffer[0] == 'S' )
            {
               // Turn is over, return false...
               retval = false;
               cont = false;
            }
            else
            {
                Console.WriteLine("\n*** Please enter an H or S and press ENTER...");
            }
         }
         catch( System.IndexOutOfRangeException )
         {
            // Didn't enter a value, so ask again
            cont = true;
         }         
      }
      return retval;
   }
}
//-------------  END OF LISTING --------------//
